# Print an optspec for argparse to handle cmd's options that are independent of any subcommand.
function __fish_tms_global_optspecs
	string join \n generate= h/help V/version
end

function __fish_tms_needs_command
	# Figure out if the current invocation already has a command.
	set -l cmd (commandline -opc)
	set -e cmd[1]
	argparse -s (__fish_tms_global_optspecs) -- $cmd 2>/dev/null
	or return
	if set -q argv[1]
		# Also print the command, so this can be used to figure out what it is.
		echo $argv[1]
		return 1
	end
	return 0
end

function __fish_tms_using_subcommand
	set -l cmd (__fish_tms_needs_command)
	test -z "$cmd"
	and return 1
	contains -- $cmd[1] $argv
end

complete -c tms -n "__fish_tms_needs_command" -l generate -r -f -a "{bash\t'',elvish\t'',fish\t'',powershell\t'',zsh\t''}"
complete -c tms -n "__fish_tms_needs_command" -s h -l help -d 'Print help'
complete -c tms -n "__fish_tms_needs_command" -s V -l version -d 'Print version'
complete -c tms -n "__fish_tms_needs_command" -f -a "config" -d 'Configure the defaults for search paths and excluded directories'
complete -c tms -n "__fish_tms_needs_command" -f -a "start" -d 'Initialize tmux with the default sessions'
complete -c tms -n "__fish_tms_needs_command" -f -a "switch" -d 'Display other sessions with a fuzzy finder and a preview window'
complete -c tms -n "__fish_tms_needs_command" -f -a "windows" -d 'Display the current session\'s windows with a fuzzy finder and a preview window'
complete -c tms -n "__fish_tms_needs_command" -f -a "kill" -d 'Kill the current tmux session and jump to another'
complete -c tms -n "__fish_tms_needs_command" -f -a "sessions" -d 'Show running tmux sessions with asterisk on the current session'
complete -c tms -n "__fish_tms_needs_command" -f -a "rename" -d 'Rename the active session and the working directory'
complete -c tms -n "__fish_tms_needs_command" -f -a "refresh" -d 'Creates new worktree windows for the selected session'
complete -c tms -n "__fish_tms_needs_command" -f -a "clone-repo" -d 'Clone repository and create a new session for it'
complete -c tms -n "__fish_tms_needs_command" -f -a "init-repo" -d 'Initialize empty repository'
complete -c tms -n "__fish_tms_needs_command" -f -a "bookmark" -d 'Bookmark a directory so it is available to select along with the Git repositories'
complete -c tms -n "__fish_tms_needs_command" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
complete -c tms -n "__fish_tms_using_subcommand config" -s p -l paths -d 'The paths to search through. Shell like expansions such as \'~\' are supported' -r
complete -c tms -n "__fish_tms_using_subcommand config" -s s -l session -d 'The default session to switch to (if available) when killing another session' -r
complete -c tms -n "__fish_tms_using_subcommand config" -l excluded -d 'As many directory names as desired to not be searched over' -r
complete -c tms -n "__fish_tms_using_subcommand config" -l remove -d 'As many directory names to be removed from exclusion list' -r
complete -c tms -n "__fish_tms_using_subcommand config" -l full-path -d 'Use the full path when displaying directories' -r -f -a "{true\t'',false\t''}"
complete -c tms -n "__fish_tms_using_subcommand config" -l search-submodules -d 'Also show initialized submodules' -r -f -a "{true\t'',false\t''}"
complete -c tms -n "__fish_tms_using_subcommand config" -l recursive-submodules -d 'Search submodules for submodules' -r -f -a "{true\t'',false\t''}"
complete -c tms -n "__fish_tms_using_subcommand config" -l switch-filter-unknown -d 'Only include sessions from search paths in the switcher' -r -f -a "{true\t'',false\t''}"
complete -c tms -n "__fish_tms_using_subcommand config" -s d -l max-depths -d 'The maximum depth to traverse when searching for repositories in search paths, length should match the number of search paths if specified (defaults to 10)' -r
complete -c tms -n "__fish_tms_using_subcommand config" -l picker-highlight-color -d 'Background color of the highlighted item in the picker' -r
complete -c tms -n "__fish_tms_using_subcommand config" -l picker-highlight-text-color -d 'Text color of the hightlighted item in the picker' -r
complete -c tms -n "__fish_tms_using_subcommand config" -l picker-border-color -d 'Color of the borders between widgets in the picker' -r
complete -c tms -n "__fish_tms_using_subcommand config" -l picker-info-color -d 'Color of the item count in the picker' -r
complete -c tms -n "__fish_tms_using_subcommand config" -l picker-prompt-color -d 'Color of the prompt in the picker' -r
complete -c tms -n "__fish_tms_using_subcommand config" -l session-sort-order -d 'Set the sort order of the sessions in the switch command' -r -f -a "{Alphabetical\t'',LastAttached\t''}"
complete -c tms -n "__fish_tms_using_subcommand config" -s h -l help -d 'Print help'
complete -c tms -n "__fish_tms_using_subcommand start" -s h -l help -d 'Print help'
complete -c tms -n "__fish_tms_using_subcommand switch" -s h -l help -d 'Print help'
complete -c tms -n "__fish_tms_using_subcommand windows" -s h -l help -d 'Print help'
complete -c tms -n "__fish_tms_using_subcommand kill" -s h -l help -d 'Print help'
complete -c tms -n "__fish_tms_using_subcommand sessions" -s h -l help -d 'Print help'
complete -c tms -n "__fish_tms_using_subcommand rename" -s h -l help -d 'Print help'
complete -c tms -n "__fish_tms_using_subcommand refresh" -s h -l help -d 'Print help'
complete -c tms -n "__fish_tms_using_subcommand clone-repo" -s h -l help -d 'Print help'
complete -c tms -n "__fish_tms_using_subcommand init-repo" -s h -l help -d 'Print help'
complete -c tms -n "__fish_tms_using_subcommand bookmark" -s d -l delete -d 'Delete instead of add a bookmark'
complete -c tms -n "__fish_tms_using_subcommand bookmark" -s h -l help -d 'Print help'
complete -c tms -n "__fish_tms_using_subcommand help; and not __fish_seen_subcommand_from config start switch windows kill sessions rename refresh clone-repo init-repo bookmark help" -f -a "config" -d 'Configure the defaults for search paths and excluded directories'
complete -c tms -n "__fish_tms_using_subcommand help; and not __fish_seen_subcommand_from config start switch windows kill sessions rename refresh clone-repo init-repo bookmark help" -f -a "start" -d 'Initialize tmux with the default sessions'
complete -c tms -n "__fish_tms_using_subcommand help; and not __fish_seen_subcommand_from config start switch windows kill sessions rename refresh clone-repo init-repo bookmark help" -f -a "switch" -d 'Display other sessions with a fuzzy finder and a preview window'
complete -c tms -n "__fish_tms_using_subcommand help; and not __fish_seen_subcommand_from config start switch windows kill sessions rename refresh clone-repo init-repo bookmark help" -f -a "windows" -d 'Display the current session\'s windows with a fuzzy finder and a preview window'
complete -c tms -n "__fish_tms_using_subcommand help; and not __fish_seen_subcommand_from config start switch windows kill sessions rename refresh clone-repo init-repo bookmark help" -f -a "kill" -d 'Kill the current tmux session and jump to another'
complete -c tms -n "__fish_tms_using_subcommand help; and not __fish_seen_subcommand_from config start switch windows kill sessions rename refresh clone-repo init-repo bookmark help" -f -a "sessions" -d 'Show running tmux sessions with asterisk on the current session'
complete -c tms -n "__fish_tms_using_subcommand help; and not __fish_seen_subcommand_from config start switch windows kill sessions rename refresh clone-repo init-repo bookmark help" -f -a "rename" -d 'Rename the active session and the working directory'
complete -c tms -n "__fish_tms_using_subcommand help; and not __fish_seen_subcommand_from config start switch windows kill sessions rename refresh clone-repo init-repo bookmark help" -f -a "refresh" -d 'Creates new worktree windows for the selected session'
complete -c tms -n "__fish_tms_using_subcommand help; and not __fish_seen_subcommand_from config start switch windows kill sessions rename refresh clone-repo init-repo bookmark help" -f -a "clone-repo" -d 'Clone repository and create a new session for it'
complete -c tms -n "__fish_tms_using_subcommand help; and not __fish_seen_subcommand_from config start switch windows kill sessions rename refresh clone-repo init-repo bookmark help" -f -a "init-repo" -d 'Initialize empty repository'
complete -c tms -n "__fish_tms_using_subcommand help; and not __fish_seen_subcommand_from config start switch windows kill sessions rename refresh clone-repo init-repo bookmark help" -f -a "bookmark" -d 'Bookmark a directory so it is available to select along with the Git repositories'
complete -c tms -n "__fish_tms_using_subcommand help; and not __fish_seen_subcommand_from config start switch windows kill sessions rename refresh clone-repo init-repo bookmark help" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
